#!/usr/bin/python3

from pygame import Surface
from .page import Page


class PageManager:
    """
    Simple class for managing "pages". A page can be used like a tab in a web browser or something else.
    """

    def __init__(self, app: any=None):
        """
        :param app: Some class that gets passed to all pages so they can all access it.
            This has no builtin functionality.
        """

        self.app = app

        self.DefaultPage = Page()  # empty page, should be overwritten

        self.current_page = "DefaultPage"

        self.page_names = []

    def draw(self, surface: Surface) -> None:
        """
        Draws the current page onto a pygame.Surface.

        :param pygame.Surface surface: The surface to draw on.
        """

        getattr(self, self.current_page).draw(surface)

    def update(self) -> None:
        """
        Updates all pages.
        """

        for page_name in self.page_names:
            page = getattr(self, page_name)
            page.update()

    def __iadd__(self, other: Page):
        self.page_names.append(type(other).__name__)
        setattr(self, type(other).__name__, other)

        return self

    def __setattr__(self, key, value):
        self.__dict__[key] = value

        if key == "current_page":
            getattr(self, self.current_page).visible = False  # tell the page that it is not visible anymore

            getattr(self, value).visible = True  # and tell the new page that it is now visible
